/*
 * file       : 9904mtr.c
 * ver        : 1.2
 * executable : 9904mtr.exe
 * commandline: 9904mtr <portnum> </s>
 *               where portnum -> LPT1 | LPT2
 *                and option /s forces a meter setup.
 * description: SA9904B pulse demonstration program, uses PM9904BPD module.
 *                      (see PM9904BPD application note on Website)
 *
 * Log        :
 *             1.0 July   2001 _hr	init release
 *             1.1 April  2002 _mb	missing phase info
 *             1.2 August 2002 _df	tidied up,
 *					added:-
 *                                      (1) calibration option
 *					(2) phase/item selection
 *					     for LCD
 *					(3) meter setup,
 *					     in case user requires other
 *					     pulserate/V_nom.
 *
 *             energy@sames.co.za
 */

//project's includes
#include <bios.h>
#include <conio.h>
#include <ctype.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "9904mtr.h"

/*Fcn: crt_user()
 * Displays user menu,
 *  displays LCD sub-menu only when its on.
 * parms: none
 * returns: nothing
 */
void crt_user(void)
{
  clrscr();

  printf("%s",Display[header]);
  printf("%s",Display[line1] );
  printf("%s",Display[line2] );
  printf("%s",Display[line3] );
  printf("%s",Display[line4] );
  printf("%s",Display[line5] );
  printf("%s",Display[line6] );
  if(crt.lcd_on)
   {
    printf("%s",Display[line7] );
    gotoxy(41,4);puts("Output to LCD:-");
    gotoxy(41,5);puts("(p) Phase");
    gotoxy(41,6);puts("(i) Item");
   }
  else printf("%s",Display[line8] );
  gotoxy(1,12);printf("%s",Display[line9] );

  crt.user=TRUE; //user screen being displayed
  CrtHideCursor();//no cursor required
}

/*Fcn: crt_update()
 * The 'live' screen, displays active/reactive energy
 *  and per phase data (Voltage/Frequency/Status bits)
 *Also redisplays user menu if required.
 * parms: none
 * returns: nothing
 */
void crt_update(void)
{
  uchar phase, //counter
	col;   //screen position
  static float f_stored[PHASE]={0,0,0},
	       v_stored[PHASE]={0,0,0};
	 float f_now[PHASE],
	       v_now[PHASE];

    if(!crt.user) crt_user();

/* test the status bits in the freq reg
 * only need to look at 1 phase, they're common.
 */
    ph_seq= (fm[0] & BIT22)?TRUE:FALSE;
    ph_err= (fm[0] & BIT21)?TRUE:FALSE;
    ph_err3=(fm[0] & BIT20)?TRUE:FALSE;
    ph_err2=(fm[0] & BIT19)?TRUE:FALSE;
    ph_err1=(fm[0] & BIT18)?TRUE:FALSE;

    if((!(ph_seq))& (!(ph_err)))
     {
      for(phase=0;phase<PHASE;phase++)
       {
	gotoxy(1+phase*20,18);
	printf("%s",Display[No_err]);
       }
     }
    if(!ph_err)
     {
      if(ph_seq)
       {
	for(phase=0;phase<PHASE;phase++)
	 {
	  gotoxy(1+phase*20,18);
	  printf("%s",Display[Swapped]);
	 }
       }
     }
    else if(ph_err)
     {
      if (ph_err1) {gotoxy(1,18); printf("%s",Display[Ph1]);}
      if (ph_err2) {gotoxy(21,18);printf("%s",Display[Ph2]);}
      if (ph_err3) {gotoxy(41,18);printf("%s",Display[Ph3]);}
     }

  gotoxy(1,14);   printf("                      ");
  gotoxy(1,14);   printf("Active :%10.3f\n",ae);
  gotoxy(41,14);  printf("                      ");
  gotoxy(41,14);  printf("Reactive :%10.3f\n",re);

/* update voltage & frequency display*/
  for(phase=0;phase<PHASE;phase++)
   {
    col=((2*phase*10)+1);

    v_now[phase]=(float)(v[phase]/CAL[phase].volt*Setup.v_nom);
    if(fabs(v_stored[phase]-v_now[phase])>CHANGE)
      v_stored[phase]=v_now[phase];
    if(fm[phase] & LAST_10)
     {
      f_now[phase]=(float)(CLOCKFREQ/256/(fm[phase] & LAST_10));
      if(fabs(f_stored[phase]-f_now[phase])>CHANGE)
	f_stored[phase]=f_now[phase];
     }
      gotoxy(col,22);printf("%s",Display[Clr_line]);
      gotoxy(col,22);printf("Volt :%10.0f\n",v_stored[phase]);
      gotoxy(col,20);printf("%s",Display[Clr_line]);
      gotoxy(col,20);printf("Freq :%3.2f\n",f_stored[phase]);
   }
}

/*Fcn: lcd_out()
 * Displays selected item/phase on module's LCD
 * parms: item & phase from caller
 * returns: nothing
 */
void lcd_out(uchar item, uchar phase)
{
  static float f_stored[PHASE]={0,0,0},
	       v_stored[PHASE]={0,0,0};
	 float f_now[PHASE],
	       v_now[PHASE];
  uchar cnt; //for loop counter

/* calculate voltage & frequency for display */
  for(cnt=0;cnt<PHASE;cnt++)
   {
    v_now[cnt]=(float)(v[cnt]/CAL[cnt].volt*Setup.v_nom);
    if(fabs(v_stored[cnt]-v_now[cnt])>CHANGE)
     v_stored[cnt]=v_now[cnt];
    f_now[cnt]=(float)(100-(fm[cnt]&0x0fffff)*50/280);
    if(fabs(f_stored[cnt]-f_now[cnt])>CHANGE)
     f_stored[cnt]=f_now[cnt];
   }

  switch(item)
  {
   case Active:
		sprintf(buf,"%1.3f",ae);
		lcd_str_2_mem(buf);
		lcd_mem_icon_off();
		lcd_mem_ph(phase);
		lcd_mem_kwh(TRUE);
		break;
   case Reactive:
		sprintf(buf,"%1.3f",re);
		lcd_str_2_mem(buf);
		lcd_mem_icon_off();
		lcd_mem_ph(phase);
		lcd_mem_kvar(TRUE);
		break;
   case Voltage:
		sprintf(buf,"%4.2f",v_stored[phase]);
		lcd_str_2_mem(buf);
		lcd_mem_icon_off();
		lcd_mem_ph(phase);
		lcd_mem_volt(TRUE);
		break;
   case Frequency:
		sprintf(buf,"%4.2f",f_stored[phase]);
		lcd_str_2_mem(buf);
		lcd_mem_icon_off();
		lcd_mem_ph(phase);
		lcd_mem_hz(TRUE);
		break;
      default :
		break;
  }
  lcd_update();
}

/*Fcn: set_thresh()
 * Updates the affected threshold(s)
 * parms: pstart & pend from caller
 * returns: nothing
 */
void set_thresh(uchar pstart, uchar pend)
{
 uchar num_phases, index;

 num_phases = pend - pstart;
 thresh.act= 0; thresh.react= 0;
 for(index = pstart; index < pend; index++)
  {
   thresh.act   += Setup.threshold*(1+CAL[index].active);
   thresh.react += Setup.threshold*(1+CAL[index].reactive);
  }
 thresh.act   /= num_phases;
 thresh.react /= num_phases;
}

/*Fcn: set_volt()
 * Calculates the cal factor(s) for volatages
 * parms: nothing
 * returns: uchar that contains missing (phase) voltage info
 */
uchar set_volt(void)
{
uchar phase,	    //counter
      status;	    //return variable
float displayed;    //temp storage

   status=0;
   for(phase=0;phase<PHASE;phase++)
    {
     displayed=((float)v[phase]/CAL[phase].volt*Setup.v_nom);
     if(displayed)
      CAL[phase].volt= CAL[phase].volt*(1+error(displayed,Setup.v_nom));
     else status|=(phase+1);//1,2,3 0|1|2|3 -> 3
    }
  return(status);
}

/*Fcn: write_cal()
 * writes present cal factors to disk
 * parms: none
 * returns: nothing
 */
void write_cal(void)
{
 uchar i; //for loop cnter
 FILE *out;

 if ( !(out = fopen(PATH,"wb")) )
   {
    fprintf(stderr, "Cannot open output file.\n");
    exit(1); //Fatal exit program
   }
  else //write to file
   {
     puts("writing data to meter.cal                             ");
     fwrite(&Setup,sizeof(Setup),1,out);
     for(i=0;i<PHASE;i++) fwrite(&CAL[i],sizeof(CAL),1,out);
     fclose(out);
   }
 delay(500); //for 'writing file' echo
}

/*Fcn: wait_key()
 * wait for keypress
 * parms: none
 * returns: nothing
 */
uchar wait_key(void)
 {
  while(!bioskey(1)) ;
  return( toupper(bioskey(0)) );
 }

/*Fcn: crt_cal()
 * does voltage cal in background then echos present cal factors
 * and allows updating of the 'active' phases
 * parms: pstart & pend from main()
 * returns: TRUE,FALSE -> success,failure
 *          When successful updates the threshold.
 */
uchar crt_cal(uchar pstart, uchar pend)
{
uchar phase,	    //counter
      buffer[16],   //display buffer
      column,       //x,y position
      line,
      p_line,	    //prompt's line
      choice; 	    //user's choice(s)
float tmp,	    //for user's input
      displayed;    //for voltage cal
const TopRow=1,     //box's 4 corners
      TopCol=2,
      BotRow=20,
      BotCol=70;
static uchar once=0, //ctrl flag for user input
	     status; //the return variable

if(!status & once) once=0;//user wants to redo cal entry

 clrscr();
 crt.user=FALSE; //user screen not displayed

 if(!once) //voltage calibration
  {
   if( set_volt() )
    {
     printf("\aHardware error not sensing voltage\nPress any key to continue\n");
     choice=wait_key();//discard key
     return(0);//retry later?
    }
  }

/* threshold correction */
  clrscr();//start afresh
  CrtShowCursor();
  CrtGotoxy(0,0);
  CrtEeop();
  CrtBox(0, CRT_NORMAL, TopRow, TopRow, BotRow, BotCol); //create box
  printf(" Present calibration values in %\n");

  CrtGotoxy(TopCol+20,TopRow+1);
  printf("Active:-");
  CrtGotoxy(TopCol+20,TopRow+5);
  printf("Reactive:-");

    for(phase=0;phase<PHASE;phase++)		//echo present cal factors
     {
      column= ((TopCol+10)+phase*20);
      line= 3;
      CrtGotoxy(column,line);

      sprintf(buffer,"Phase%d",(phase+1) );	//display phase number
      puts(buffer);

      CrtGotoxy(column,++line);
      printf("%5.3f\n",CAL[phase].active*100.0); //display in % -> *100.0

      column= ((TopCol+10)+phase*20);
      line= 7;
      CrtGotoxy(column,line);
      sprintf(buffer,"Phase%d",(phase+1) );	//display phase number
      puts(buffer);

      CrtGotoxy(column,++line);
      printf("%5.3f\n",CAL[phase].reactive*100.0);
     }

/* create space for prompts */
  CrtGotoxy(TopCol,++line); p_line=++line;
  for(phase=0;phase<68;phase++)  printf("=");   //draw line
  CrtGotoxy(TopCol,p_line+1);
  for(phase=0;phase<68;phase++)  printf("=");   //draw line

  CrtGotoxy(1,BotRow+1);
  printf("<Esc> returns to live screen");//permanent prompt

  if(!once) //get the user's input
   {
    once=1;
    CrtGotoxy(TopCol+5,p_line);
    printf("Update the above");
    textcolor(GREEN);
    cprintf(" a"); printf("ctive or ");
    cprintf("r"); printf("e-active");
    printf(" values.");
    textcolor(LIGHTGRAY);//normal white text colour

    choice=wait_key();//wait for user's choice
    if ((choice == 'A') | (choice=='R'))  //else jump out
     {
      for(phase=pstart;phase<pend;phase++)
       {
	CrtGotoxy(TopCol+5,(p_line+3)+phase);
	printf("Phase%d=",phase+1);
	switch(choice)
	{
	 case 'A':
		  scanf("%f",&tmp);
		  fflush(stdin);
		  CAL[phase].active=(tmp/100.0);//format inputs, we need <= 1.00
		  break;
	 case 'R':
		  scanf("%f",&tmp);
		  fflush(stdin);
		  CAL[phase].reactive=(tmp/100.0);
		  break;
	 default :
		  break;
	}
       }
      set_thresh(pstart, pend); //update the thresholds
      status=TRUE; //to allow echo of user's inputs
     }
    else //not A|R, possibly Esc?
     {
      if(choice==ESC) status=FALSE;//return to crt_update()
       else status=TRUE;
      once=0;
     }
   }

  else if(once&status)  //echo the user's input
   {
     CrtGotoxy(TopCol+1,p_line);//can Esc without saving
     printf("<w> to write cal file...");
     choice=wait_key();//wait for user's choice

    switch(choice)
     {
      case ESC: //Esc key pressed, return to crt_update()
	       status=FALSE;
	       break;
      case 'W': //w key pressed, write cal file
	       CrtEeol(TopCol+1,p_line);  //ensure write_cal's prompt
	       CrtGotoxy(TopCol+1,p_line);//at right place
	       write_cal();
	       break;
      default : //does nothing, remain on this screen until Esc|w
	       break;
     }
   }
 return(status);
}

/*Fcn: defaults()
 * defaults all calibration factors
 * parms: none
 * returns: nothing
 */
void defaults(void)
{
 uchar cnt; //for loop counter

  for(cnt=0;cnt<PHASE;cnt++)
   {
    CAL[cnt].active=   0.0;
    CAL[cnt].reactive= 0.0;
    CAL[cnt].volt  =   700;
    }
}

/*Fcn: setup_meter()
 * allows user to set up meter to their specs,
 * requests from user
 *		nominal voltage
 *		maximum load current
 *		pulserate
 * parms: none
 * returns: nothing
 */
void setup_meter(void)
{
 double epc, 		  //energy per count
	epp; 		  //energy per LED pulse
 ulong threshold; 	  //the calculated threshold
 const TopRow=8,	  //box's 4 corners
       TopCol=20,
       BotRow=23,
       BotCol=60;
 uchar line,col,	  //to keep track of where we are
       dummy;   	  //for reading key press

 clrscr();
 printf("\aMeter not setup.\nPress any key to enter setup screen\n");
  dummy=wait_key();//wait for keypress

 CrtShowCursor();
 CrtGotoxy(0,0);
 CrtEeop();
 CrtBox(3, CRT_NORMAL, TopRow, TopCol, BotRow, BotCol); //create box

 printf("Please enter:\n");	//user prompt
 printf(" nominal voltage,\n");
 printf(" maximum load current\n");
 printf(" and meter's pulserate\n");

 line=TopRow+1;  //origin for input fields
 col=TopCol+2;

 CrtGotoxy(col,line);
 printf("V_NOM=");scanf("%f",&Setup.v_nom);
 CrtGotoxy(col+20,line);printf("[V]\n");
 CrtPokew(1,30,(CRT_NORMAL*256+251));//echo ok to user
 ++line;

 CrtGotoxy(col,line);
 printf("I_max=");scanf("%f",&Setup.i_max);
 CrtGotoxy(col+20,line);printf("[A]\n");
 CrtPokew(2,30,(CRT_NORMAL*256+251));
 ++line;

 CrtGotoxy(col,line);
 printf("pulserate=");scanf("%f",&Setup.p_rate);
 CrtGotoxy(col+20,line);printf("[p/kWh]\n");
 CrtPokew(3,30,(CRT_NORMAL*256+251));

 line+=2;
 epc=(Setup.v_nom*Setup.i_max)/RATE;
 CrtGotoxy(col,line);   printf("epc= %f\n",epc);
 CrtGotoxy(col+20,line);printf("[Ws]\n");

 ++line;
 epp=(1000.*3600.)/Setup.p_rate;
 CrtGotoxy(col,line);   printf("epp= %f\n",epp);
 CrtGotoxy(col+20,line);printf("[Ws/pulse]\n");

 line+=2;
 Setup.threshold=(ulong) epp/epc;
 CrtGotoxy(col,line);   printf("threshold= %ld\n",Setup.threshold);
 CrtGotoxy(col+20,line);printf("[counts]\n");

 defaults();	   //defaults cal factors
 CrtHideCursor();
 textcolor(CYAN+BLINK);
 CrtGotoxy(TopCol,BotRow+1); cprintf("press any key to continue");
  dummy=wait_key();//wait for keypress
 textcolor(LIGHTGRAY);//reset text colour
}

/*Fcn: load_cal()
 * read cal file, if(!exist) request meter setup from user
 *  and set to defaults
 * parms: none
 * returns: nothing
 */
void load_cal(void)
{
  FILE *in; //cal file handle
  uchar cnt; //for loop counter

 if ( !(in = fopen(PATH,"rb")) ) setup_meter(); //setup the meter
  else //read from file
   {
     fread(&Setup,sizeof(Setup),1,in);
     for(cnt=0;cnt<PHASE;cnt++) fread(&CAL[cnt],sizeof(CAL),1,in);
     fclose(in);
   }
  set_thresh(0,PHASE);//all phases
  set_volt();/*ignore the return val,
	      *we'll catch error in calib fcn
	      */
}

void main (int argc,char *argv[])
{
  uchar lcd_phase, lcd_item;

  uchar key, first,
	phase, pstart, pend;
  uint  portnum, cnt;

  long ai;    				//active integrator sum 3 and compares against threshold
  long an[3]; 				//active now 1 of 3 phases
  long ap[3]; 				//active previous 1 of 3 phases old register value
  int a_led_pulses; 			//amount of counted pulses that will increment 1 kW

  long ri;    				//re-active integrator sum 3  and compares against threshold
  long rn[3]; 				//re-active now 1 of 3 phases
  long rp[3]; 				//re-active previous 1 of 3 phases old register value
  int r_led_pulses; 			//amount of counted pulses that will increment 1 kva

  int a_led_ontime, r_led_ontime;  	//LED on-time counter

  uchar busy=1, d;
  uchar slow_seq[6] = {2,3,6,7,10,11};
  int cnt2;
  int vcnt, fcnt, seq_cnt;
  unsigned int i,j, status;

  clrscr(); //start afresh
  if(argc>=2)
   {
    portnum=atoi(argv[1]);
    port= (portnum == 1)?LPT1:LPT2;
    if(argc==3 && strcmp(argv[3],"/s")) setup_meter(); //setup the meter
     else load_cal(); //read cal file
   }
  else
   {
    printf("\nUsage : 9904mtr <portnum> </s>\n");
    printf("        <portnum>   = 1 or 2 (lpt port number)\n");
    printf("        < /s>       = Force a meter setup\n");
    exit(0);
   }

  lcd_phase=0; lcd_item=0;//phase 1 and active energy

  vcnt=0;
  fcnt=0;
  seq_cnt=0;
  cnt2=0;
  cnt=0;
  portval=0;
  portval2=0;
  pstart=0; pend=1;
  ae=0;
  re=0;
  a_led_pulses=0;
  r_led_pulses=0;

  first=1;
  ai=0;
  ri=0;
  a_led_ontime=0;
  r_led_ontime=0;

  pstart=0; pend=3;//default 'sum of 3phases' mode

/*init crt bitfields*/
  crt.cal        = FALSE;//not being calibrated
  crt.user       = FALSE;//user screen not being displayed
  crt.display_on = TRUE; //update screen is on
  crt.lcd_on     = FALSE;//LCD is off

  spi_out(portval | LED1 | LED2 | LED3); //init the LPT port
  outport(port+2,(portval2 & ~PCVDD1 & ~PCVDD2 | PCVDD3 & ~PCVDD4));
  spi_reset();

  lcd_test();				//LCD test display

  while(busy)
  {
    if(bioskey(1))
    {
      key = toupper(bioskey(0));

      switch(key)
      {
       case 'Q': //exit from program
		 busy = 0;
		 break;
       case 'P': //step LCD's phase
		 lcd_phase++;
		 if(lcd_phase>2) lcd_phase=0;
		 break;
       case 'I': //step LCD's item
		 lcd_item++;
		 if(lcd_item>Frequency) lcd_item=Active;
		 break;
       case 'C': //toggle cal value display
		 crt.cal^=1;
		 break;
       case 'D': //toggle the CRT display On/Off
		 crt.display_on^=1;
		 break;
       case 'L': //toggle the LCD on the module On/Off
		 crt.lcd_on^=1;
		 crt.user=0; //force redisplay of user screen
		 if(!crt.lcd_on) lcd_clear();
		 break;
       case 'S': //pulses based on sum of the 3 phases
		 pstart=0; pend=3;
		 set_thresh(pstart,pend);
		 break;
       case '3': //select only phase 3
		 pstart=2; pend=3;
		 set_thresh(pstart,pend);
		 break;
       case '2': //select only phase 2
		 pstart=1; pend=2;
		 set_thresh(pstart,pend);
		 break;
       case '1': //select only phase 1
		 pstart=0; pend=1;
		 set_thresh(pstart,pend);
		 break;
       default :
		 break;
      }
      first=1;
    }

    for(phase=pstart;phase<pend;phase++) //read the power registers as fast as possible
    {
      spi_reset();
      spi_cs();
      spi_start_read(phase*4);
      an[phase] = spi_read_register();
      rn[phase] = spi_read_register();
    }
    cnt++;

    for(phase=pstart;phase<pend&&first==0;phase++)
    {
      ai+= convert_24bits(an[phase]-ap[phase]);
      ri+= convert_24bits(rn[phase]-rp[phase]);
    }

    if(first==0)
    {
      if(fabs(ai)>thresh.act)		//check threshold and generate active pulses
      {
	if (ai<0)
	  ai+=thresh.act;
	else
	  ai-=thresh.act;
	spi_out(portval & ~LED3);
	a_led_ontime=ONTIME;
	a_led_pulses++;
	if(a_led_pulses>A_PULSES_PER_WH)
	{ ae+=0.001; a_led_pulses=0; }
      }

      if(fabs(ri)>thresh.react)		//check threshold and generate re-active pulses
      {
	if (ri<0)
	  ri+=thresh.react;
	else
	  ri-=thresh.react;
	spi_out(portval & ~LED2);
	r_led_ontime=ONTIME;
	r_led_pulses++;
	if(r_led_pulses>R_PULSES_PER_VAH)
	{ re+=0.001; r_led_pulses=0; }
      }

      if(cnt>2)				//rough counter to time LED ON duration
      {
	if(a_led_ontime>0)		//active LED time out counter
	  a_led_ontime--;
	else
	  spi_out(portval | LED3);

	if(r_led_ontime>0)        	//re-active LED time out counter
	  r_led_ontime--;
	else
	  spi_out(portval | LED2);
	cnt2++;
	cnt = 0;
      }

      if((cnt2>1) && (crt.display_on))      //read the voltage and frequency registers mainly for display
      {
	spi_reset();
	spi_cs();
	spi_start_read(slow_seq[seq_cnt]);
	switch(seq_cnt)
	{
	case 0:
	case 2:
	case 4:
	  v[vcnt] = spi_read_register();
	  vcnt++;
	  break;
	case 1:
	case 3:
	case 5:
	  fm[fcnt] = spi_read_register();
	  fcnt++;
	  break;
	}
	seq_cnt++;
	if(vcnt>2) vcnt=0;
	if(fcnt>2) fcnt=0;
	if(seq_cnt>5) seq_cnt=0;
	cnt2=0;
	  if(!crt.cal) crt_update();
	   else  crt.cal= crt_cal(pstart,pend);  //call cal with pstart & pend
	if(crt.lcd_on) lcd_out(lcd_item,lcd_phase);
      }
    }
    ap[0]=an[0]; ap[1]=an[1]; ap[2]=an[2];
    rp[0]=rn[0]; rp[1]=rn[1]; rp[2]=rn[2];
    first=0;
  }
}