/*
 * file       : 9904mtr.h
 * ver        : 1.0
 *
 * description: Header file for PM9904BPD module's DOS software.
 *
 * Log        :
 *             1.0 August  2002 _df	init coding
 *
 *             energy@sames.co.za
 */
#ifndef __METER_H
#define __METER_H

//i'm a lazy typist
typedef unsigned char uchar;
typedef unsigned int  uint;
typedef unsigned long ulong;

//general constants
#define TRUE	1
#define FALSE	0
#define ESC	0x1b //Escape key
#define CHANGE  2.5  //if(change in freq/volt>CHANGE) update display

//chip-related constants
#define RATE		 320000		//register update rate
#define CLOCKFREQ	 3579545	//clock frequency
#define BIT22		 0x0400000	//Frequency register bitmasks, bit22 ... bit18
#define BIT21		 0x0200000
#define BIT20		 0x0100000
#define BIT19		 0x080000
#define BIT18		 0x040000
#define LAST_10		 0x0003FF 	//mask for last 10 bits in 24 bit number

//parallel port constants
#define LPT1		 0x378
#define LPT2		 0x378//0x278, why i don't know
#define CE_b		 0x10  		//D4 on port  pin 6 SA8807 CE LCD
#define CLK_b 		 0x20  		//D5 on port  pin 7
#define DO_b  		 0x40  		//D6 on port  pin 8
#define CS_b  		 0x80  		//D7 on port  pin 9 SA990x CS Meter
#define DI_b  		 0x20  		//Paper out  port+1  pin 12
#define PCTrig 		 0x01  		//D0 on port  pin 2
#define LED1   		 0x02  		//D1 on port  pin 3
#define LED2   		 0x04  		//D2 on port  pin 4
#define LED3   		 0x08  		//D3 on port  pin 5
#define PCVDD1 		 0x01  		//D0  on port+2 pin 1
#define PCVDD2 		 0x02  		//D1  on port+2 pin 14
#define PCVDD3 		 0x04  		//D2  on port+2 pin 16  This one is inverted !
#define PCVDD4 		 0x08  		//D3  on port+2 pin 17

//measurement constants
#define PHASE		 3		//3phase device
#define ONTIME 		 1
#define A_PULSES_PER_WH  10
#define R_PULSES_PER_VAH 10
#define PATH "\\meter.cal" 		//path to cal file

//macro to calculate error
#define error(measured,expected) (float)(measured-expected)/expected

//global variables
long delay_time=1; 	     /* history delay the spi readings for slow Opto-couplers,
			      * not needed with HP2631 opto couplers.
			      */
uint  port;   		     //lpt port addr
uchar portval;  	     //global variable containing status of the LPT port
uchar portval2; 	     //global variable containing status of LPT+2 port
long  fm[PHASE]; 	     //mains frequency 1 of 3 phases old register value
long  v[PHASE];  	     //mains voltage   1 of 3 phases old register value
float ae,re; 		     //accumulated energy in kWh and kVarh
uchar ph_err, 		     //flags for status bits in frequency register
      ph_err1,
      ph_err2,
      ph_err3,
      ph_seq=0;

char *Display[17]= {	     //various user interface prompts
		      "               ",
		      "No Phase Error ",
		      "Phases swapped ",
		      "Phase 1 missing",
		      "Phase 2 missing",
		      "Phase 3 missing",
		      "SAMES                 P M  9 9 0 4 B P D           M O D U L E              2002\n\n",
		      "   (1) Phase 1 only\n",
		      "   (2) Phase 2 only\n",
		      "   (3) Phase 3 only\n",
		      "   (s) Sum of three phases\n",
		      "   (c) Calibration values\n",
		      "   (d) Display On\\Off \n",
		      "   (l) LCD     ON \n\n",
		      "   (l) LCD     OFF\n\n",
		      "   (q) Quit the program"
		     };

/*Look Up Table for prompts*/
enum LUT {
	  Clr_line,//crt_update
	  No_err,
	  Swapped,
	  Ph1,
	  Ph2,
	  Ph3,
	  header,  //crt_user
	  line1,
	  line2,
	  line3,
	  line4,
	  line5,
	  line6,
	  line7,
	  line8,
	  line9,
	  };

/*Look Up Table for LCD*/
enum LCD  {
	    Active,
	    Reactive,
	    Voltage,
	    Frequency,
	  };

/* calibration variables */
struct
      {
       uchar cal        : 1; //control calibration screen
       uchar user       : 1; //user screen control
       uchar display_on : 1; //CRT display  On/Off
       uchar lcd_on     : 1; //module's LCD On/Off
      }crt;

struct
      {
       float active;  //active register threshold correction
       float reactive;//reactive register threshold correction
       uint  volt;    //voltage correction
      }CAL[PHASE];

struct	       //thresholds for active and reactive registers
      {
       long act;
       long react;
      }thresh;

struct
      {
	float v_nom;     //applied volatge      [V]
	float i_max;	 //max load current     [A]
	float p_rate;    //meter pulse-rate     [p/kWh]
	ulong threshold; //calculated threshold [counts]
      }Setup;

#include "pc_spi.c"
#include "pc_lcd.c"
#include "crt.c"

#endif //__METER_H








