/*
 * File      : crt.c
 * ver       : 1.0
 * desription:
 *	low level functions for creating 2D DOS GUI
 *
 * Log       :
 *           1.0 1993-6-9 DeonF init coding
 *
 */

#include "crt.h"

//character sets
const char *ul = "\xc9\xda\xdb\xd5\xd6"; /* ""; */
const char *ur = "\xbb\xbf\xdb\xb8\xb7"; /* "۸"; */
const char *bl = "\xc8\xc0\xdb\xd4\xd3"; /* ""; */
const char *br = "\xbc\xd9\xdb\xbe\xbd"; /* "۾"; */
const char *hz = "\xcd\xc4\xdb\xcd\xc4"; /* ""; */
const char *vt = "\xba\xb3\xdb\xb3\xba"; /* "۳"; */


void CrtGotoxy(int x, int y) //home cursor-> (0,0)
{
  gotoxy(x+1, y+1);
}

void CrtEeol(unsigned here, unsigned rcol)//erase from cursor to end of line
{
 int cnt,line;

 line=wherex();
  for(cnt=here;cnt<rcol;cnt++)
  { CrtGotoxy(++here,line); puts(" ");}
}

void CrtEeop(void)//erase from cursor to end of page
{
  int xx, yy, y;
  struct text_info info;

  gettextinfo(&info);
  clreol();
  yy = wherey();
  xx = wherex();
  for (y = yy+1; y <= info.screenheight; y++)
  {
    gotoxy(1, y);
    clreol();
  }
  gotoxy(xx, yy);
}

//draw box's outline
void CrtBox(int type, int attr, unsigned trow, unsigned lcol,
				unsigned brow, unsigned rcol)
{
  unsigned cul, cur, cbl, cbr, chz, cvt;
  unsigned uattr = attr * 256;
  int x, y;

//drawing characters based on type and attribute
  if (type < 0 || type > 4) type = 0;
  cul = uattr + (uchar)ul[type];
  cur = uattr + (uchar)ur[type];
  cbl = uattr + (uchar)bl[type];
  cbr = uattr + (uchar)br[type];
  chz = uattr + (uchar)hz[type];
  cvt = uattr + (uchar)vt[type];

//display corners
  CrtPokew(trow, lcol, cul);
  CrtPokew(trow, rcol, cur);
  CrtPokew(brow, lcol, cbl);
  CrtPokew(brow, rcol, cbr);

//display outline
  for (x = lcol + 1; x < rcol; x++)
  {
    CrtPokew(trow, x, chz);
    CrtPokew(brow, x, chz);
  }
  for (y = trow + 1; y < brow; y++)
  {
    CrtPokew(y, lcol, cvt);
    CrtPokew(y, rcol, cvt);
  }
}

//poke uchar into display
void CrtPokew(int row, int col, unsigned attrchar)
{
  col++;
  row++;
  puttext(col, row, col, row, &attrchar);
}

//copy display buffer bytes to buffer at save pointer
void CrtPeekBox(void *save, unsigned trow, unsigned lcol,
			    unsigned brow, unsigned rcol)
{
  gettext(lcol + 1, trow + 1, rcol + 1, brow + 1, save);
}

//paint the box
void CrtFillBox(unsigned attrchar, unsigned trow, unsigned lcol,
				   unsigned brow, unsigned rcol)
{
  int i;

  while (trow <= brow)
  {
    for (i = lcol; i <= rcol; i++)
      CrtPokew(trow, i, attrchar);
    trow++;
  }
}

//copy data saved by CrtPeekBox back to display
void CrtPokeBox(void *save, unsigned trow, unsigned lcol,
			    unsigned brow, unsigned rcol)
{
  puttext(lcol + 1, trow + 1, rcol + 1, brow + 1, save);
}


void CrtHideCursor(void) //just wrapper fcn
{
  _setcursortype(_NOCURSOR);
}


void CrtShowCursor(void) //just wrapper fcn
{
  _setcursortype(_NORMALCURSOR);
}

void CrtScroll(int lines, unsigned ulrow, unsigned ulcol,
			  unsigned lrrow, unsigned lrcol, unsigned attr)
{
  union REGS regs;

  if (lines < 0)
  {
    regs.h.ah = 7;     //scroll down
    lines = -lines;
  }
  else
  regs.h.ah = 6;     //scroll up
  regs.h.ch = ulrow;
  regs.h.cl = ulcol;
  regs.h.dh = lrrow;
  regs.h.dl = lrcol;
  regs.h.bh = attr;
  if (ulrow >= lrrow)
    regs.h.al = 0;     //clr window if top >= bottom row
  else
    regs.h.al = lines; //else scroll the specified number of lines
  int86(VIDEO, &regs, &regs);
}


int CrtGetMode(void) //current display mode
{
  struct text_info info;

  gettextinfo(&info);
  return info.currmode;
}
