/*
 * file       : pc_lcd.c
 * ver        : 1.0
 *
 * description: Control of the PM9904BPD's LCD.
 *
 * Log        :
 *             1.0 July  2001 _hr	init release
 */

#define MAX_DIGITS 8

char map[13]={						// charmap start at - to 9 ASCII 45
  	      0x20, // -
  	      0x08, // .
  	      0x50, // /
  	      0xd7, // 0
  	      0x06, // 1
  	      0xe3, // 2 
  	      0xa7, // 3 
  	      0x36, // 4 
  	      0xb5, // 5 
  	      0xf5, // 6
  	      0x07, // 7 
  	      0xf7, // 8 
  	      0xb7  // 9
	     }; 
char disp[12]; 
char buf[12];

void lcd_mem_ph1(char status)
{
  if(status) disp[10]|=0x40; 
   else disp[10]&=~0x40;
}

void lcd_mem_ph2(char status)
{
  if(status) disp[10]|=0x80; 
   else disp[10]&=~0x80;
}

void lcd_mem_ph3(char status)
{
  if(status) disp[9]|=0x80;
   else disp[9]&=~0x80;
}

void lcd_mem_ph(char status)
{
  switch(status)
  {
  case 0:
	 disp[10]|=0x40;
	 break;
  case 1:
	 disp[10]|=0x80;
	 break;
  case 2:
	 disp[9]|=0x80;
	 break;
  default:
	 break;
/*
    {
      disp[9]|=0x80;
      disp[10]|=0x80;
      disp[10]|=0x40;
    }
*/
  }
}

void lcd_mem_icon_off(void)
{
  disp[0]=0x0;
  disp[9]=0;
  disp[10]=0; 
  disp[11]=0;
}

void lcd_mem_cos(char status)
{
  if(status)  disp[0]|=0x10; 
   else disp[0]&=~0x10;
}

void lcd_mem_volt(char status)
{
  if(status)  disp[9]|=0x01; 
   else disp[9]&=~0x01;
}

void lcd_mem_hz(char status)
{
  if(status)  disp[10]|=0x20; 
   else disp[10]&=~0x20;
}

void lcd_mem_amp(char status)
{
  if(status) disp[9]|=0x02; 
   else disp[9]&=~0x02;
}

void lcd_mem_kvar(char status)
{
  if(status)
  { 
   disp[10]|=0x10; 
   disp[9]|=0x07; 
  }
  else
  { 
   disp[10]&=~0x10; 
   disp[9]&=~0x07;
  }
}

void lcd_mem_kwh(char status)
{
  if(status) disp[10]|=0x0b;  
   else  disp[10]&=~0x0b;
}


void lcd_str_2_mem(char *number_str)			// put a character string into display memory
{
  char lcd_mem_adr, character,i;

  for(i=1;i<9;i++)
    disp[i]=0;

  lcd_mem_adr=8;
  i=strlen(number_str);
  do
  {
    i--;
    character=number_str[i];
    disp[lcd_mem_adr]|=map[character-'-'];
    if(character != '.')
      lcd_mem_adr--;
  }
  while(i>0);
  // pad with spaces
  for(i=lcd_mem_adr;i>0;i--)
    disp[i]=0;
}

void lcd_update(void)
{
  char i;
  spi_reset();
  spi_ce();
  spi_adr_LCD(0x00);
  for(i=0;i<12;i++)
    spi_data_LCD(disp[i]);
}

void lcd_clear(void)
{
  char i;
  spi_reset();
  spi_ce();
  spi_adr_LCD(0x00);
  for(i=0;i<12;i++)
  {
    spi_data_LCD(0);
    disp[i]=0;
  }
}

void lcd_test(void)
{
  char i;
  spi_reset();
  spi_ce();
  spi_adr_LCD(0x00);
  for(i=0;i<12;i++)
    spi_data_LCD(0xff);
  delay(300);
  spi_reset();
  spi_ce();
  spi_adr_LCD(0x00);
  for(i=0;i<12;i++)
    spi_data_LCD(0x00);
}
