/*
 * file       : pc_spi.c
 * ver        : 1.0
 *
 * description: Communication with the PM9904BPD board.
 *
 * Log        :
 *             1.0 July  2001 _hr	init release
 */

#define CLK_m	 ~CLK_b					// Mask definitions for the parallele port
#define DO_m	 ~DO_b
#define DI_m     ~DI_b
#define CS_m     ~CS_b
#define byte_mask 0x80  				//1000 0000



long convert_24bits(long value)	       //this function checks for wrapping of the 24 bitregister values
{
  if (labs(value) > 0x7FFFFF)
  {
    if (value > 0)
      value = (16777216-value) * (-1);
    else
      value = (16777216+value) * (1);
  }
  return(value);
}

void spi_wait(void)		       //this function will wait for the specified period
{
  int  ti,txy;
  for (ti=0;ti<delay_time;ti++)
  {
   txy++;
  } ;
}

void spi_out(unsigned char value)      //put value on the parallel port
{
  portval = value;
  outport(LPT2,~portval);              //inverts for Opto couplers
}

void spi_clock(void)		       //this function will pulse CLK pin of spi port
{
  spi_wait();
  spi_out(portval | CLK_b);
  spi_wait();
  spi_out(portval ^ CLK_b);
}

void spi_PCTrig(void)		       //this function will set the PC trigger pin
{
  spi_out(portval & (~PCTrig));
  spi_wait();
}

void spi_ClearPCTrig(void)		//this function will clear the PC trigger pin
{
  spi_out(portval | PCTrig);
}

void spi_cs(void)		       //this function will set CSs pin of spi port
{
  spi_out(portval | CS_b);
}

void spi_reset(void)		       //all bits of the port are made 0
{
  spi_out(portval & ~CLK_b);
  spi_out(portval & ~DO_b);
  spi_out(portval & ~CS_b);
  spi_out(portval & ~CE_b);
}

void spi_start_read(int value)	       //all bits of the port are made 0
{
  int counter;
  int and_mask,mask;
  mask = 0x100;
  value = (value | 0x180 );            //this put the header for the adress 110---
  for ( counter = 0; counter <9; counter++)
  {
    and_mask = (mask & value);

    if (and_mask == mask)
    {
      spi_out(portval | DO_b);
      spi_clock();
    }
    else
    {
      spi_out(portval & DO_m);
      spi_clock();
    }
    value = value << 1;
  }
}

unsigned long spi_read_register(void)
							/* This function will read 24 bits of a register
							 * the register to be red would be indicated by the
							 * preceding spi_start_read function
							 */
{
  int counter;
  unsigned long fromport;

  fromport = 0 ;

  for ( counter = 0; counter < 24; counter++)
  {
    fromport = fromport << 1; //make room for first bit
    spi_clock();

    if (((inportb(LPT2+1)) & DI_b) == 0)//inverting the MISO
      fromport = fromport | 0x01;
  }
  return(fromport);
}

void spi_ce(void)  		       //this function will set CE pin of spi port
{
  spi_out(portval | CE_b);
}

void spi_adr_LCD(unsigned char adr)
{
  unsigned char cnt, mask;
  mask=0x80;
  adr|=0x80;   // write mode to LCD
  for(cnt=0;cnt<8;cnt++)
  {
    if(adr&mask)
    {
      spi_out(portval | DO_b);
      spi_clock();
    }
    else
    {
      spi_out(portval & ~DO_b);
      spi_clock();
    }
    mask>>=1;
  }
}

void spi_data_LCD(unsigned char adr)
{
  unsigned char cnt, mask;
  mask=0x80;   //write mode to LCD
  for(cnt=0;cnt<8;cnt++)
  {
    if(adr&mask)
    {
      spi_out(portval | DO_b);
      spi_clock();
    }
    else
    {
      spi_out(portval & ~DO_b);
      spi_clock();
    }
    mask>>=1;
  }
}
